@@ -13,13 +13,14 @@
 # with this software. If not, see
 # <http://creativecommons.org/publicdomain/zero/1.0/>.
 
-script_lang = (('latn', ('dflt', 'TRK ')), ('arab', ('dflt', 'ARA ', 'URD ', 'SND ')), ('DFLT', ('dflt',)))
+from __future__ import print_function
 
-from sortsmill import ffcompat as fontforge
-from sortsmill import psMat
-import sys
+import fontforge
+import psMat
 import os
 
+from tempfile import NamedTemporaryFile
+
 def cleanAnchors(font):
     """Removes anchor classes (and associated lookups) that are used only
     internally for building composite glyph."""
@@ -103,54 +104,64 @@ def validateGlyphs(font):
 
         glyph.round()
 
-        # Hack, OTS rejects ligature carets!
-        glyph.lcarets = ()
-
 def updateInfo(font, version):
     from datetime import datetime
 
-    version = "%07.3f" % float(version)
+    version = "%07.3f" % version
     font.version = font.version % version
     font.copyright = font.copyright % datetime.now().year
-    for name in font.sfnt_names:
-        if name[0] == "Arabic (Egypt)":
-            if name[1] == "Version":
-                version = version.replace(".", "\xD9\xAB")
-                font.appendSFNTName(name[0], name[1], name[2] % version)
-            elif name[1] == "Copyright":
-                font.appendSFNTName(name[0], name[1], name[2] % datetime.now().year)
 
-def mergeFeatures(font, feafile):
-    """Merges feature file into the font while making sure mark positioning
-    lookups (already in the font) come after kerning lookups (from the feature
-    file), which is required by Uniscribe to get correct mark positioning for
-    kerned glyphs."""
+def generateFeatureString(font, lookup):
+    with NamedTemporaryFile() as tmp:
+        font.generateFeatureFile(tmp.name, lookup)
+        fea = tmp.read()
+        return fea.decode("utf-8")
 
-    # create dummy glyphs used for some coding hacks
-    for i in [1, 2]:
-        dummy = font.createChar(-1, "dummy%s" %i)
-        dummy.width = 0
-
-    oldfea = font.generateFeatureString()
+def generateFeatures(font, feafile):
+    """Generates feature text by merging feature file with mark positioning
+    lookups (already in the font) and making sure they come after kerning
+    lookups (from the feature file), which is required by Uniscribe to get
+    correct mark positioning for kerned glyphs."""
 
+    oldfea = ""
     for lookup in font.gpos_lookups:
-        font.removeLookup(lookup)
+        oldfea += generateFeatureString(font, lookup)
 
-    for lookup in font.gsub_lookups:
+    for lookup in font.gpos_lookups + font.gsub_lookups:
         font.removeLookup(lookup)
 
     # open feature file and insert the generated GPOS features in place of the
     # placeholder text
-    fea = open(feafile)
-    fea_text = fea.read()
+    with open(feafile, 'rb') as fea:
+        fea_text = fea.read().decode("utf-8")
     fea_text = fea_text.replace("{%anchors%}", oldfea)
-    fea.close()
 
-    # now merge it into the font
-    font.mergeFeatureString(fea_text)
+    bases = [g.glyphname for g in font.glyphs() if g.glyphclass != "mark"]
+    marks = [g.glyphname for g in font.glyphs() if g.glyphclass == "mark"]
+    carets = {g.glyphname: g.lcarets for g in font.glyphs() if any(g.lcarets)}
+    gdef = []
+    gdef.append("@GDEFBase = [%s];" % " ".join(bases))
+    gdef.append("@GDEFMark = [%s];" % " ".join(marks))
+    gdef.append("table GDEF {")
+    gdef.append("  GlyphClassDef @GDEFBase, , @GDEFMark, ;")
+    for k, v in carets.items():
+        gdef.append("  LigatureCaretByPos %s %s;" % (k, " ".join(map(str, v))))
+    gdef.append("} GDEF;")
 
-def generateFont(font, outfile):
-    flags  = ("opentype", "dummy-dsig", "round", "omit-instructions", "no-mac-names")
+    fea_text += "\n".join(gdef)
+
+    return fea_text
+
+def generateFont(font, feafile, feastring, outfile):
+    from fontTools.feaLib.builder import addOpenTypeFeaturesFromString
+    from fontTools.ttLib import TTFont
+
+    fea = generateFeatures(font, feafile)
+    fea += feastring
+
+    flags = ["opentype", "dummy-dsig", "round", "omit-instructions"]
+    if fontforge.version() > '20170805':
+        flags.append("no-mac-names")
 
     font.selection.all()
     font.correctReferences()
@@ -161,10 +172,22 @@ def generateFont(font, outfile):
 
     font.generate(outfile, flags=flags)
 
-def drawOverUnderline(font, name, uni, glyphclass, pos, thickness, width):
+    try:
+        ttfont = TTFont(outfile)
+        addOpenTypeFeaturesFromString(ttfont, fea)
+        ttfont.save(outfile)
+    except:
+        with NamedTemporaryFile(delete=False) as tmp:
+            tmp.write(fea.encode("utf-8"))
+            print("Failed! Inspect temporary file: %r" % tmp.name)
+            os.remove(outfile)
+        raise
+
+
+def drawOverline(font, name, uni, pos, thickness, width):
     glyph = font.createChar(uni, name)
     glyph.width = 0
-    glyph.glyphclass = glyphclass
+    glyph.glyphclass = "mark"
 
     pen = glyph.glyphPen()
 
@@ -176,68 +199,46 @@ def drawOverUnderline(font, name, uni, glyphclass, pos, thickness, width):
 
     return glyph
 
-def makeOverUnderline(font, over=True, under=True, o_pos=None, u_pos=None):
+def makeQuranSajdaLine(font, pos):
     # test string:
     # صِ̅فْ̅ ̅خَ̅ل̅قَ̅ ̅بًّ̅ صِ̲فْ̲ ̲خَ̲ل̲قَ̲ ̲بِ̲
 
     thickness = font.uwidth # underline width (thickness)
-    minwidth = 100.0
-
-    if not o_pos:
-        o_pos = font.os2_typoascent
-
-    if not u_pos:
-        u_pos = font.upos - thickness # underline pos
+    minwidth = 100
 
     # collect glyphs grouped by their widths rounded by 100 units, we will use
     # them to decide the widths of over/underline glyphs we will draw
     widths = {}
     for glyph in font.glyphs():
-        if glyph.glyphclass == 'baseglyph' and glyph.unicode != 0xFDFD:
-            width = round(glyph.width/100) * 100
+        u = glyph.unicode
+        if ((u < 0) or (0x0600 <= u <= 0x06FF) or u == ord(" ")) \
+        and glyph.width > 0:
+            width = round(glyph.width / minwidth) * minwidth
             width = width > minwidth and width or minwidth
             if not width in widths:
                 widths[width] = []
             widths[width].append(glyph.glyphname)
 
-    if over:
-        o_base = drawOverUnderline(font, 'uni0305', 0x0305, 'mark', o_pos, thickness, 500)
+    base = 'uni0305'
+    drawOverline(font, base, 0x0305, pos, thickness, 500)
 
-    if under:
-        u_base = drawOverUnderline(font, 'uni0332', 0x0332, 'mark', u_pos, thickness, 500)
-
-    markset = "%s %s" %(over and o_base.glyphname or "", under and u_base.glyphname or "")
-
-    font.addMarkSet("OverUnderSet", markset)
-
-    context_lookup_name = 'OverUnderLine'
-    font.addLookup(context_lookup_name, 'gsub_contextchain', ('OverUnderSet'), (('mark', script_lang),), font.gsub_lookups[-1])
+    fea = []
+    fea.append("@OverSet = [%s];" % base)
+    fea.append("feature mark {")
+    fea.append("  lookupflag UseMarkFilteringSet @OverSet;")
 
     for width in sorted(widths.keys()):
         # for each width group we create an over/underline glyph with the same
         # width, and add a contextual substitution lookup to use it when an
         # over/underline follows any glyph in this group
+        name = 'uni0305.%d' % width
+        drawOverline(font, name, -1, pos, thickness, width)
+        fea.append("  sub [%s] %s' by %s;" % (" ".join(widths[width]), base, name))
 
-        single_lookup_name = str(width)
+    fea.append("} mark;")
 
-        font.addLookup(single_lookup_name, 'gsub_single', (), (), font.gsub_lookups[-1])
-        font.addLookupSubtable(single_lookup_name, single_lookup_name + '1')
-
-        if over:
-            o_name = 'uni0305.%d' % width
-            o_glyph = drawOverUnderline(font, o_name, -1, 'mark', o_pos, thickness, width)
-            o_base.addPosSub(single_lookup_name + '1', o_name)
-
-        if under:
-            u_name = 'uni0332.%d' % width
-            u_glyph = drawOverUnderline(font, u_name, -1, 'mark', u_pos, thickness, width)
-            u_base.addPosSub(single_lookup_name + '1', u_name)
-
-        context = "%s %s" %(over and o_base.glyphname or "", under and u_base.glyphname or "")
-
-        rule = '| [%s] [%s] @<%s> | ' %(" ".join(widths[width]), context, single_lookup_name)
-
-        font.addContextualSubtable(context_lookup_name, context_lookup_name + str(width), 'coverage', rule)
+    fea = "\n".join(fea)
+    return fea
 
 def centerGlyph(glyph):
     width = glyph.width
@@ -262,52 +263,41 @@ def subsetFont(font, glyphnames, similar=False):
                 glyphnames.append(ref[0])
         else:
             if name not in reported:
-                print 'Font ‘%s’ is missing glyph: %s' %(font.fontname, name)
+                print('Font ‘%s’ is missing glyph: %s' %(font.fontname, name))
                 reported.append(name)
 
+    # Prune kerning classes
+    for lookup in font.gpos_lookups:
+        for subtable in font.getLookupSubtables(lookup):
+            if font.isKerningClass(subtable):
+                first, second, offsets = font.getKerningClass(subtable)
+                first  = [[n for n in c if n in glyphnames] for c in first]
+                second = [[n for n in c if n in glyphnames] for c in second if c]
+                second.insert(0, None)
+                if all([first, second]):
+                    font.alterKerningClass(subtable, first, second, offsets)
+                else:
+                    font.removeLookupSubtable(subtable)
+
     # remove everything else
     for glyph in font.glyphs():
         if glyph.glyphname not in glyphnames:
             font.removeGlyph(glyph)
 
-def buildComposition(font, glyphnames):
-    newnames = []
+def subsetFontFT(path, unicodes):
+    from fontTools.ttLib import TTFont
+    from fontTools import subset
 
-    font.addLookup("Latin composition", 'gsub_ligature', (), (('ccmp', script_lang),))
-    font.addLookupSubtable("Latin composition", "Latin composition subtable")
+    font = TTFont(path)
 
-    import unicodedata
-    for name in glyphnames:
-        u = fontforge.unicodeFromName(name)
-        if 0 < u < 0xfb00:
-            decomp = unicodedata.decomposition(unichr(u))
-            if decomp:
-                base = decomp.split()[0]
-                mark = decomp.split()[1]
-                if not '<' in base:
-                    nmark = None
-                    nbase = None
+    options = subset.Options()
+    options.set(layout_features='*', name_IDs='*', name_languages='*',
+        notdef_outline=True, glyph_names=True)
+    subsetter = subset.Subsetter(options=options)
+    subsetter.populate(unicodes=unicodes)
+    subsetter.subset(font)
 
-                    for g in font.glyphs():
-                        if g.unicode == int(base, 16):
-                            nbase = g.glyphname
-                        if g.unicode == int(mark, 16):
-                            nmark = g.glyphname
-
-                    if not nbase:
-                        nbase = "uni%04X" % int(base, 16)
-                    if not nmark:
-                        nmark = "uni%04X" % int(mark, 16)
-
-                    if nbase in font and nmark in font:
-                        font[name].addPosSub("Latin composition subtable", (nbase, nmark))
-
-                    if base not in glyphnames:
-                        newnames.append(nbase)
-                    if mark not in glyphnames:
-                        newnames.append(nmark)
-
-    return newnames
+    font.save(path)
 
 def makeNumerators(font):
     digits = ("zero", "one", "two", "three", "four", "five", "six", "seven",
@@ -325,15 +315,15 @@ def makeNumerators(font):
             numr.addReference(small.glyphname, psMat.translate(0, 550))
             numr.width = small.width
 
-def mergeLatin(font, feafile, italic=False, glyphs=None, quran=False):
-    styles = {"Regular": "regular",
-              "Slanted": "italic",
-              "Bold": "bold",
-              "BoldSlanted": "bolditalic"}
+def mergeLatin(font, italic=False, glyphs=None, quran=False):
+    styles = {"Regular": "Regular",
+              "Slanted": "Italic",
+              "Bold": "Bold",
+              "BoldSlanted": "BoldItalic"}
 
     style = styles[font.fontname.split("-")[1]]
 
-    latinfile = "amirilatin-%s.sfdir" %style
+    latinfile = "AmiriLatin-%s.sfdir" %style
 
     from tempfile import mkstemp
     tmpfont = mkstemp(suffix=os.path.basename(latinfile).replace("sfdir", "sfd"))[1]
@@ -406,9 +396,8 @@ def mergeLatin(font, feafile, italic=False, glyphs=None, quran=False):
     if not quran:
         # we want our ring above and below in Quran font only
         for name in ("uni030A", "uni0325"):
-            font[name].clear()
+            font.removeGlyph(name)
 
-        latinglyphs += buildComposition(latinfont, latinglyphs)
     subsetFont(latinfont, latinglyphs)
 
     digits = ("zero", "one", "two", "three", "four", "five", "six", "seven",
@@ -421,9 +410,9 @@ def mergeLatin(font, feafile, italic=False, glyphs=None, quran=False):
     # upright so it works reasonably with bot scripts
     if italic:
         if "bold" in style:
-            upright = fontforge.open("sources/latin/amirilatin-bold.sfdir")
+            upright = fontforge.open("sources/latin/AmiriLatin-Bold.sfdir")
         else:
-            upright = fontforge.open("sources/latin/amirilatin-regular.sfdir")
+            upright = fontforge.open("sources/latin/AmiriLatin-Regular.sfdir")
 
         shared = ("exclam", "quotedbl", "numbersign", "dollar", "percent",
                   "quotesingle", "asterisk", "plus", "colon", "semicolon",
@@ -476,20 +465,12 @@ def mergeLatin(font, feafile, italic=False, glyphs=None, quran=False):
             rtl.useRefsMetrics(pname)
 
     # copy kerning classes
-    kern_lookups = {}
+    fea = ""
     if not quran:
         for lookup in latinfont.gpos_lookups:
-            kern_lookups[lookup] = {}
-            kern_lookups[lookup]["subtables"] = []
-            kern_lookups[lookup]["type"], kern_lookups[lookup]["flags"] = latinfont.getLookupInfo(lookup)[:2]
-            for subtable in latinfont.getLookupSubtables(lookup):
-                if latinfont.isKerningClass(subtable):
-                    kern_lookups[lookup]["subtables"].append((subtable, latinfont.getKerningClass(subtable)))
+            fea += generateFeatureString(latinfont, lookup)
 
-    for lookup in latinfont.gpos_lookups:
-        latinfont.removeLookup(lookup)
-
-    for lookup in latinfont.gsub_lookups:
+    for lookup in latinfont.gpos_lookups + latinfont.gsub_lookups:
         latinfont.removeLookup(lookup)
 
     latinfont.save(tmpfont)
@@ -498,8 +479,10 @@ def mergeLatin(font, feafile, italic=False, glyphs=None, quran=False):
     font.mergeFonts(tmpfont)
     os.remove(tmpfont)
 
-    if not quran:
-        buildComposition(font, latinglyphs)
+    # This is often used for visible space, so they better be the same width.
+    if "periodcentered" in font:
+        font["periodcentered"].width = font["space"].width
+        centerGlyph(font["periodcentered"])
 
     # add Latin small and medium digits
     for name in digits:
@@ -525,35 +508,11 @@ def mergeLatin(font, feafile, italic=False, glyphs=None, quran=False):
             medium.width = 900
             centerGlyph(medium)
 
-    for lookup in kern_lookups:
-        font.addLookup(lookup,
-                kern_lookups[lookup]["type"],
-                kern_lookups[lookup]["flags"],
-                (('kern', script_lang),)
-                )
-
-        for subtable in kern_lookups[lookup]["subtables"]:
-            first = []
-            second = []
-            offsets = subtable[1][2]
-
-            # drop non-existing glyphs
-            for new_klasses, klasses in ((first, subtable[1][0]), (second, subtable[1][1])):
-                for klass in klasses:
-                    new_klass = []
-                    if klass:
-                        for name in klass:
-                            if name in font:
-                                new_klass.append(name)
-                    new_klasses.append(new_klass)
-
-            # if either of the classes is empty, don’t bother with the subtable
-            if any(first) and any(second):
-                font.addKerningClass(lookup, subtable[0], first, second, offsets)
+    return fea
 
 def makeSlanted(infile, outfile, feafile, version, slant):
 
-    font = makeDesktop(infile, outfile, feafile, version, False, False)
+    font = makeDesktop(infile, outfile, feafile, version, False)
 
     # compute amout of skew, magic formula copied from fontforge sources
     import math
@@ -565,9 +524,10 @@ def makeSlanted(infile, outfile, feafile, version, slant):
         font.removeGlyph(glyph)
 
     font.selection.all()
-    punct = ("period", "guillemotleft", "guillemotright", "braceleft", "bar",
-             "braceright", "bracketleft", "bracketright", "parenleft",
-             "parenright", "slash", "backslash", "brokenbar", "uni061F")
+    punct = ("exclam", "period", "guillemotleft", "guillemotright",
+             "braceleft", "bar", "braceright", "bracketleft", "bracketright",
+             "parenleft", "parenright", "slash", "backslash", "brokenbar",
+             "uni061F", "dot.1", "dot.2")
 
     for name in punct:
         font.selection.select(["less"], name)
@@ -579,20 +539,13 @@ def makeSlanted(infile, outfile, feafile, version, slant):
     font.fullname += " Slanted"
     if font.weight == "Bold":
         font.fontname = font.fontname.replace("Bold", "BoldSlanted")
-        font.appendSFNTName("Arabic (Egypt)", "SubFamily", "عريض مائل")
         font.appendSFNTName("English (US)",   "SubFamily", "Bold Slanted")
     else:
         font.fontname = font.fontname.replace("Regular", "Slanted")
-        font.appendSFNTName("Arabic (Egypt)", "SubFamily", "مائل")
 
-    mergeLatin(font, feafile, italic=skew)
+    fea = mergeLatin(font, italic=skew)
     makeNumerators(font)
-
-    # we want to merge features after merging the latin font because many
-    # referenced glyphs are in the latin font
-    mergeFeatures(font, feafile)
-
-    generateFont(font, outfile)
+    generateFont(font, feafile, fea, outfile)
 
 def scaleGlyph(glyph, amount):
     """Scales the glyph, but keeps it centered around its original bounding
@@ -616,17 +569,20 @@ def scaleGlyph(glyph, amount):
         glyph.width = width
 
 def makeQuran(infile, outfile, feafile, version):
-    font = makeDesktop(infile, outfile, feafile, version, False, False)
+    font = makeDesktop(infile, outfile, feafile, version, False)
 
     # fix metadata
     font.fontname = font.fontname.replace("-Regular", "Quran-Regular")
     font.familyname += " Quran"
     font.fullname += " Quran"
 
+    sample = "بِسۡمِ ٱللَّهِ ٱلرَّحۡمَٰنِ ٱلرَّحِیمِ ۝١ ٱلۡحَمۡدُ لِلَّهِ رَبِّ ٱلۡعَٰلَمِینَ ۝٢"
+    font.appendSFNTName('English (US)', 'Sample Text', sample)
+
     digits = ("zero", "one", "two", "three", "four", "five", "six",
               "seven", "eight", "nine")
 
-    mergeLatin(font, feafile, glyphs=digits, quran=True)
+    fea = mergeLatin(font, glyphs=digits, quran=True)
 
     punct = ("period", "guillemotleft", "guillemotright", "braceleft", "bar",
              "braceright", "bracketleft", "bracketright", "parenleft",
@@ -653,13 +609,11 @@ def makeQuran(infile, outfile, feafile, version):
                  "TwoDots.a", "ThreeDots.a", "vTwoDots.a"):
         scaleGlyph(font[mark], 0.9)
 
+    # create overline glyph to be used for sajda line, it is positioned
+    # vertically at the level of the base of waqf marks
+    fea += makeQuranSajdaLine(font, font[0x06D7].boundingBox()[1])
+
     quran_glyphs = []
-    for glyph in font.glyphs():
-        if glyph.name.startswith("dummy"):
-            quran_glyphs.append(glyph.name)
-
-    mergeFeatures(font, feafile)
-
     quran_glyphs += digits
     quran_glyphs += punct
     quran_glyphs += ("space",
@@ -689,8 +643,8 @@ def makeQuran(infile, outfile, feafile, version):
             "uni202E", "uni202F", "uni25CC", "uniFD3E", "uniFD3F", "uniFDFA",
             "uniFDFD")
     quran_glyphs += ("uni030A", "uni0325") # ring above and below
-
-    subsetFont(font, quran_glyphs, True)
+    quran_glyphs += ["uni0305"] # overline
+    unicodes = [font[n].unicode for n in quran_glyphs]
 
     # set font ascent to the highest glyph in the font so that waqf marks don't
     # get truncated
@@ -698,21 +652,18 @@ def makeQuran(infile, outfile, feafile, version):
     # the offset relative to em-size in the former and font bounds in the
     # later, but we want both to be relative to font bounds
     ymax = 0
-    for glyph in font.glyphs():
+    for name in quran_glyphs:
+        glyph = font[name]
         bb = glyph.boundingBox()
         if bb[-1] > ymax:
             ymax = bb[-1]
 
     font.os2_typoascent = font.hhea_ascent = ymax
 
-    # create overline glyph to be used for sajda line, it is positioned
-    # vertically at the level of the base of waqf marks
-    overline_pos = font[0x06D7].boundingBox()[1]
-    makeOverUnderline(font, under=False, o_pos=overline_pos)
+    generateFont(font, feafile, fea, outfile)
+    subsetFontFT(outfile, unicodes)
 
-    generateFont(font, outfile)
-
-def makeDesktop(infile, outfile, feafile, version, latin=True, generate=True):
+def makeDesktop(infile, outfile, feafile, version, generate=True):
     font = fontforge.open(infile)
     font.encoding = "UnicodeFull" # avoid a crash if compact was set
 
@@ -721,90 +672,33 @@ def makeDesktop(infile, outfile, feafile, version, latin=True, generate=True):
     # remove anchors that are not needed in the production font
     cleanAnchors(font)
 
-    #makeOverUnderline(font)
-
     # sample text to be used by font viewers
     sample = 'صِفْ خَلْقَ خَوْدٍ كَمِثْلِ ٱلشَّمْسِ إِذْ بَزَغَتْ يَحْظَىٰ ٱلضَّجِيعُ بِهَا نَجْلَاءَ مِعْطَارِ.'
 
-    for lang in ('Arabic (Egypt)', 'English (US)'):
-        font.appendSFNTName(lang, 'Sample Text', sample)
-
-    if latin:
-        mergeLatin(font, feafile)
-        makeNumerators(font)
-
-        # we want to merge features after merging the latin font because many
-        # referenced glyphs are in the latin font
-        mergeFeatures(font, feafile)
-
+    font.appendSFNTName('English (US)', 'Sample Text', sample)
 
     if generate:
-        generateFont(font, outfile)
+        fea = mergeLatin(font)
+        makeNumerators(font)
+        generateFont(font, feafile, fea, outfile)
     else:
         return font
 
-def usage(extramessage, code):
-    if extramessage:
-        print extramessage
-
-    message = """Usage: %s OPTIONS...
-
-Options:
-  --input=FILE          file name of input font
-  --output=FILE         file name of output font
-  --features=FILE       file name of features file
-  --version=VALUE       set font version to VALUE
-  --slant=VALUE         autoslant
-
-  -h, --help            print this message and exit
-""" % os.path.basename(sys.argv[0])
-
-    print message
-    sys.exit(code)
-
 if __name__ == "__main__":
-    #if fontforge.version() < min_ff_version:
-    #    print "You need FontForge %s or newer to build Amiri fonts" %min_ff_version
-    #    sys.exit(-1)
+    import argparse
+    parser = argparse.ArgumentParser(description="Build Amiri fonts.")
+    parser.add_argument("--input", metavar="FILE", required=True, help="input font to process")
+    parser.add_argument("--output", metavar="FILE", required=True, help="ouput font to write")
+    parser.add_argument("--features", metavar="FILE", required=True, help="feature file to include")
+    parser.add_argument("--version", type=float, required=True, help="font version")
+    parser.add_argument("--slant", type=float, required=False, help="font slant")
+    parser.add_argument("--quran", action='store_true', required=False, help="build Quran variant")
 
-    import getopt
-    try:
-        opts, args = getopt.gnu_getopt(sys.argv[1:],
-                "h",
-                ["help", "input=", "output=", "features=", "version=", "slant=", "quran"])
-    except getopt.GetoptError, err:
-        usage(str(err), -1)
+    args = parser.parse_args()
 
-    infile = None
-    outfile = None
-    feafile = None
-    version = None
-    slant = False
-    quran = False
-
-    for opt, arg in opts:
-        if opt in ("-h", "--help"):
-            usage("", 0)
-        elif opt == "--input": infile = arg
-        elif opt == "--output": outfile = arg
-        elif opt == "--features": feafile = arg
-        elif opt == "--version": version = arg
-        elif opt == "--slant": slant = float(arg)
-        elif opt == "--quran": quran = True
-
-    if not infile:
-        usage("No input file specified", -1)
-    if not outfile:
-        usage("No output file specified", -1)
-
-    if not version:
-        usage("No version specified", -1)
-    if not feafile:
-        usage("No features file specified", -1)
-
-    if slant:
-        makeSlanted(infile, outfile, feafile, version, slant)
-    elif quran:
-        makeQuran(infile, outfile, feafile, version)
+    if args.slant:
+        makeSlanted(args.input, args.output, args.features, args.version, args.slant)
+    elif args.quran:
+        makeQuran(args.input, args.output, args.features, args.version)
     else:
-        makeDesktop(infile, outfile, feafile, version)
+        makeDesktop(args.input, args.output, args.features, args.version)
